!#************************************************************************
!# E.S.O. - 3.6m Upgrade project
!#
!# who        when      what
!# ---------- --------  ---------------------------------------------
!# fpatat     29/07/1998  created
!# fpatat     07/08/1998  I[nteractive] / A[uthomatic] option for average
!#                        estimate added.
!# fpatat     17/08/1998  Few screen outputs added
!# pleisy     28/01/2000  Automatic cuts for MOS FlatFields
!# pleisy     04/02/2000  Improve the MEAN/MAX determination
!# msterzik   05/10/2000  Typo in stat/ima parameters removed
!# rathreya   22/03/2001  Increased minimum Y for spectroscopic mode (YminS) 
!#                        from 10 to 400 to avoid the zeroeth-order
!#
!# kbanse     090722      add verification checks   (EXTLastMean.prg)
!# kbanse     100407	  last modif
!#************************************************************************
!#   NAME
!#   EXTLastMean
!# 
!#   SYNOPSIS
!#   EXTLastMean imagename Flag
!# 
!#   DESCRIPTION
!#   Computes the Mean/Max level of the last acquired image.
!#   It is called from the EFOSClib.tcl trough PCO. 
!#    
!#   FILES
!#
!#   ENVIRONMENT
!#   PCO must be running together with a background MIDAS session
!#
!#   RETURN VALUES
!#   Average value is written in the 1st position of the OUTPUTR
!#   MIDAS keyword (REAL).
!#
!#   CAUTIONS
!#
!#   EXAMPLES
!#
!#   SEE ALSO
!#
!#   BUGS     
!#
!#------------------------------------------------------------------------
!#
define/par P1 ? I "Frame name :"
define/par P2 ? C "Estimation option :"
define/par P3 100 I "Window used [100] :"
!
write/out " "
write/out "Image:  {P1} received"
write/out "option: {P2} received"
write/out "Option:  AI  (Automatic Image) "
write/out "Option:  AS  (Automatic Spectrum) "
write/out " "
!
defi/loc Xmin/r/1/1	{{P1},START(1)}
defi/loc Ymin/r/1/1	{{P1},START(2)}
defi/loc stepX/r/1/1	{{P1},NPIX(1)}
defi/loc stepY/r/1/1	{{P1},NPIX(2)}
defi/loc binX/r/1/1	{{P1},STEP(1)}
defi/loc binY/r/1/1	{{P1},STEP(2)}
defi/loc Xmax/r/1/1	0
defi/loc Ymax/r/1/1	0
Xmax = {Xmin} + ({stepX}-1)*{binX}
Ymax = {Ymin} + ({stepY}-1)*{binY}
defi/loc choix1/C/1/1	{P2(1:1)}
defi/loc choix2/C/1/1	{P2(2:2)}
defi/loc wind/I/1/1	{P3}
defi/loc xl/r/1/1	0
defi/loc xh/r/1/1	0
defi/loc yl/r/1/1	0
defi/loc yh/r/1/1	0
defi/loc sig/r/1/1	0
defi/loc cut/R/1/4	100,64000,0,0
defi/loc level/R/1/2	.1,-.1	! level of 2% agreement
defi/loc stat/R/1/4	0
defi/loc count/I/1/1	0
defi/loc XminS/r/1/1	200
defi/loc XmaxS/r/1/1	2000
defi/loc YminS/r/1/1	1200
defi/loc YmaxS/r/1/1	2000
defi/loc countdif/I/1/1	0
defi/loc Intens/R/1/1	0
defi/loc Intens0/R/1/1	0
defi/loc ImaxX/I/1/2	0,0
defi/loc ImaxY/I/1/2	0,0
defi/loc diffX/I/1/1	0
defi/loc diffY/I/1/1	0
def/loc exst/i/1/1      0

! Reset the outputr values
outputr(1) = 0
outputr(2) = 0
outputr(3) = 0
outputr(8) = 0
!
exst = m$existk("P3")
if {exst} .ne. 0 then 
  if {P3} .ge. 200 then
    YminS = {P3}
  endif
endif

write/out 
write/out "YMIN = {YminS}"
write/out 

!
SET/FORMAT F8.3
if choix1 .eq. "A" then
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	! Valid for Plain or Polarimetry MASK images
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 	if choix2 .eq. "I" then
  		xl = {Xmin} + 0.02 * {stepX}
  		xh = {Xmin} + 0.98 * {stepX}
  		yl = {Ymin} + 0.02 * {stepY}
  		yh = {Ymin} + 0.98 * {stepY}
  		write/out " "
  		write/out "CCD statistics over 95% of the image ({xl},{yl})-({xh},{yh}) "
! 
                write/keyw rrefdat 22.6,22.6,1011.4,1011.4
                rcorr(1) = xl
                rcorr(2) = yl
                rcorr(3) = xh
                rcorr(4) = yh
                @@ kcompare lmean010:rcorr rrefdat 1,4 0.01
! 
! 
! 
		STATima:
  		set/midas output=yes
		define/local winH/i/1/1 20
		define/local winW/i/1/1 20

		rebin/lin {P1} a0001 {winW},{winH}
  		stat/ima a0001 

	write/keyw rcorr 112.86,28909.3,1113.07,424.531,43.0743,2344.2
	write/keyw rcorr/r/8/5 1119.12,282.251,256.0,112.927,1185.67
	@@ kcompare lmean020:rcorr outputr 1,3 0.1 4,5 0.001 8,11 0.001
	@@ kcompare lmean021:rcorr outputr 12,12 0.01
  	sig = M$sqrt({outputr(3)})
	rcorr(1) = sig
	rrefdat(1) = 33.363
	@@ kcompare lmean022:rrefdat rcorr 1,1 0.001

	set/midas output=yes
	! Over 60000 gives the opportunuity to still try, with 0 template is aborted
  		if outputr(3) .GE. 60000 then
      		   write/out " Image almost or fully saturated  !!! "
		   set/midas output=no
    		   goto FINima
                endif
         	if outputr(1) .EQ. 0 then
      		   outputr(3) = 0
      		   write/out " oversaturated? "
		   set/midas output=no
    		   goto FINima
    		   endif
		set/midas output=yes
  		stat(1) = 1-({outputr(3)}/{outputr(8)})
  		count = {count} + 1
  		write/out "Stat over the domain {cut(1)} {cut(2)} No {count} ... MEDIAN/MEAN variation = {stat(1)}% "
  		write/out "Mean = {outputr(3)} ; Median = {outputr(8)} ; Sigma = {sig} ; and Maximum =  {outputr(2)} "
  		!if stat(1) .GT. {level(1)} .OR. stat(1) .LT. {level(2)} then
    			!cut(1) = {outputr(3)}-2*{sig}
    			!cut(2) = {outputr(2)}
    			!goto STATima
  		!endif
                ! read/key outputr
                set/midas output=yes
		FINima:
!	endif
!	if choix2 .eq. "S" then

	elseif choix2 .eq. "S" then
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		! Valid for Long Slit or MOS/Polarimetry MASK spectra
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    		write/out "Zone used ({XminS},{YminS})-({XmaxS},{YmaxS}) "
    		write/out "Create a BDF file TEMPx ... "
    		indisk {P1} TEMPx

		write/out "Finding the true maximum position ... not a bad pixel"

		FINDmax:

    		countdif = {countdif} + 1
    		ImaxX(1) = ImaxX(2)
    		ImaxY(1) = ImaxY(2)
    		Intens0 = {cut(2)}
    		set/midas output=yes
    		stat/ima TEMPx [{XminS},{YminS}:{XmaxS},{YmaxS}] #512 {cut(1)},{cut(2)}
		copy/ii TEMPx o0001
		load o0001 scale=-2 cuts=f,ihap
		extract/image z0001 = o0001 [{XminS},{YminS}:{XmaxS},{YmaxS}]
		write/out "o0001" 

		define/local winH/i/1/1 20
		define/local winW/i/1/1 20

		rebin/lin z0001 a0001 {winW},{winH}
		write/out "z0001" 

		!!orig!! filter/median o0001 a0001 {winW},{winH}
		define/local strx/i/1/1 0
		define/local stry/i/1/1 0
		define/local binxx/i/1/1 0
		define/local binyy/i/1/1 0
		define/local nx/i/1/1   0
		define/local ny/i/1/1   0
		define/local max/r/1/1  0
		define/local xw/r/1/1   0
		define/local yw/r/1/1   0
		define/local lowCornerX/i/1/1 0
		define/local lowCornerY/i/1/1 0
		define/local height/i/1/1     0
		define/local width/i/1/1      0
		define/local upCornerX/i/1/1  0
		define/local upCornerY/i/1/1  0
		define/local avoid/i/1/1      0
		define/local winFac/i/1/1     0

		strx =  {a0001,START(1)}
		stry =  {a0001,START(2)}
		binxx =  {a0001,STEP(1)}
		binyy =  {a0001,STEP(2)}

		write/out "strx = {strx}"
		write/out "stry = {stry}"
		write/out "binxx = {binxx}"
		write/out "binyy = {binyy}"


		statistics/image a0001 


		if outputr(2) .le. 60000 then

		  max =   outputr(2)
		  xw = ( {outputi(4)} - 1 ) * {binxx} + {strx}
		  yw = ( {outputi(5)} - 1 ) * {binyy} + {stry}
		  write/out "max =  {max}"
		  write/out "xw = {xw}"
		  write/out "yw = {yw}"

		  !  draw/cross {xw},{yw} F 2 3 

		  ! lowCornerX = xw - winW * binxx
		  ! lowCornerY = yw - winH * binyy
		  ! height     = ( 2 * winH + 1 ) * binyy 
		  ! width      = ( 2 * winW + 1 ) * binxx 

		  lowCornerX = xw - winW 
		  lowCornerY = yw - winH  
		  write/out 
  		  write/out "low corner X= {lowCornerX}"
  		  write/out "low corner Y= {lowCornerY}"
  		  write/out 
  		  if lowCornerX .le. strx then
		     lowCornerX = strx +1
		  endif
		  if lowCornerY .le. stry then
		     lowCornerY = stry +1
		  endif
		  write/out "low corner X= {lowCornerX}"
		  write/out "low corner Y= {lowCornerY}"
		  write/out 

		  width      = ( 2 * winW + 1 ) 
		  height     = ( 2 * winH + 1 ) 
		  upCornerX  = lowCornerX + width
		  upCornerY  = lowCornerY + height

		  write/key outputr/r/1/20 0 ALL

		  outputr(1) = lowCornerX
		  outputr(2) = lowCornerY
		  outputr(3) = max
		  outputr(4) = height
		  outputr(5) = width

		  clear/channel overlay 0
		  draw/rectangle {lowCornerX},{lowCornerY},{upCornerX},{upCornerY} F F 3

		  writ/out "Region: [{lowCornerX},{lowCornerY}:{upCornerX},{upCornerY}]"
		  writ/out "Region size: {width}, {height}"


		else

  		  outputr(1) = -1
  		  outputr(2) = -1
  		  outputr(3) = -1
  		  outputr(4) = -1
  		  outputr(5) = -1

  		  writ/out "No suitable region: Max over 60000"
  
		endif
        else 
  	outputr(3) = 0
  	write/out "**************************************"
  	write/out "**************************************"
  	write/out " Cases M or P NOT anymore supported !!! " 
  	write/out "**************************************"
  	write/out "**************************************"
	endif
else
  	outputr(3) = 0
  	write/out "**************************************"
  	write/out "**************************************"
  	write/out " Case different from A NOT supported !!! " 
  	write/out "**************************************"
  	write/out "**************************************"
endif

!
write/out
write/out "Mean/Max level taken :" {outputr(3)}
write/out
if choix2 .eq. "S" then
  write/out " CLEANING : Remove temporary files ... "
  $rm TEMP?*
endif

return




